﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Perfusjonssytem
{
    public partial class ManualForm : Form
    {
        PerfusionForm perfusionForm;
        List<CheckBox> chkChannels; 

        internal PictureBox PbArduinoStatusMan { get => pbArduinoStatusMan; set => pbArduinoStatusMan = value; }
        public ManualForm(PerfusionForm pf)
        {
            InitializeComponent();
            this.TopLevel = false;

            perfusionForm = pf;

            chkChannels = new List<CheckBox>();
            chkChannels.Add(chkChannel1);
            chkChannels.Add(chkChannel2);
            chkChannels.Add(chkChannel3);
            chkChannels.Add(chkChannel4);

            COM.DisconnectedDel += DisableOnDisconnect;
            COM.DisconnectedDel += EnableSequence;
            COM.ConnectedDel += EnableOnConnect;
            COM.ConnectedDel += EnableSequenceIfAllClosed;

        }

        private void TryOpen(int channelNum)
        {
            ArduinoResult result = COM.TrySending(COM.Encode(channelNum, true));

            if (result != ArduinoResult.Valid)
            {
                MessageBox.Show("Failed to open the channel", "Communication error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                chkChannels[channelNum - 1].Checked = false;
            }
            else
            {
                chkChannels[channelNum - 1].Checked = true;
                DisableSequence();
            }
        }
        private void TryClose(int channelNum)
        {
            ArduinoResult result = COM.TrySending(COM.Encode(channelNum, false));

            if (result != ArduinoResult.Valid)
            {
                MessageBox.Show("Failed to close the channel", "Communication error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                chkChannels[channelNum - 1].Checked = true;
            }
            else
            {
                chkChannels[channelNum - 1].Checked = false;
                EnableSequenceIfAllClosed();
            }
        }

        private void DisableSequence()
        {
            perfusionForm.BtnSequence.Enabled = false;
        }
        private void EnableSequence()
        {
            perfusionForm.BtnSequence.Enabled = true;
        }

        private void EnableSequenceIfAllClosed()
        {
            for (int i = 0; i < chkChannels.Count; i++)
            {
                if (chkChannels[i].Checked == true) return;
            }
            perfusionForm.BtnSequence.Enabled = true;
        }

        private void chkChannel1_Click(object sender, EventArgs e)
        {
            if (chkChannel1.Checked)
            {
                if (chkSingleChannelMode.Checked) CloseAll();
                chkChannel1.FlatAppearance.MouseOverBackColor = chkChannel1.FlatAppearance.CheckedBackColor;
                TryOpen(1);
            }
            else
            {
                chkChannel1.FlatAppearance.MouseOverBackColor = chkChannel1.BackColor;
                TryClose(1);
            }
        }

        private void chkChannel2_Click(object sender, EventArgs e)
        {
            if (chkChannel2.Checked)
            {
                if (chkSingleChannelMode.Checked) CloseAll();
                chkChannel2.FlatAppearance.MouseOverBackColor = chkChannel2.FlatAppearance.CheckedBackColor;
                TryOpen(2);
            }
            else
            {
                chkChannel2.FlatAppearance.MouseOverBackColor = chkChannel2.BackColor;
                TryClose(2);
            }
        }

        private void chkChannel3_Click(object sender, EventArgs e)
        {
            if (chkChannel3.Checked)
            {
                if (chkSingleChannelMode.Checked) CloseAll();
                chkChannel3.FlatAppearance.MouseOverBackColor = chkChannel3.FlatAppearance.CheckedBackColor;
                TryOpen(3);
            }
            else
            {
                chkChannel3.FlatAppearance.MouseOverBackColor = chkChannel3.BackColor;
                TryClose(3);
            }
        }

        private void chkChannel4_Click(object sender, EventArgs e)
        {
            if (chkChannel4.Checked)
            {
                if (chkSingleChannelMode.Checked) CloseAll();
                chkChannel4.FlatAppearance.MouseOverBackColor = chkChannel4.FlatAppearance.CheckedBackColor;
                TryOpen(4);
            }
            else
            {
                chkChannel4.FlatAppearance.MouseOverBackColor = chkChannel4.BackColor;
                TryClose(4);
            }
        }

        private void btnOpenAll_Click(object sender, EventArgs e)
        {
            OpenAll();
        }

        private void btnCloseAll_Click(object sender, EventArgs e)
        {
            CloseAll();
        }
        private void OpenAll()
        {
            for (int i = 1; i <= chkChannels.Count; i++)
            {
                if (!chkChannels[i - 1].Checked)
                {
                    TryOpen(i);
                }
            }
        }
        public void CloseAll()
        {
            for (int i = 1; i <= chkChannels.Count; i++)
            {
                if (chkChannels[i - 1].Checked)
                {
                    TryClose(i);
                }
            }
        }

        private void chkSingleChannelMode_Click(object sender, EventArgs e)
        {
            if (chkSingleChannelMode.Checked)
            {
                chkSingleChannelMode.FlatAppearance.MouseOverBackColor = chkSingleChannelMode.FlatAppearance.CheckedBackColor;
                CloseAll();
                btnOpenAll.Enabled = false;
            }
            else
            {
                chkSingleChannelMode.FlatAppearance.MouseOverBackColor = chkSingleChannelMode.BackColor;
                btnOpenAll.Enabled = true;
            }
        }

        private void DisableOnDisconnect()
        {
            for (int i = 0; i < chkChannels.Count; i++)
            {
                chkChannels[i].Checked = false;
                chkChannels[i].Enabled = false;
            }
            btnCloseAll.Enabled = false;
            btnOpenAll.Enabled = false;
        }
        private void EnableOnConnect()
        {
            for (int i = 0; i < chkChannels.Count; i++)
            {
                chkChannels[i].Checked = false;
                chkChannels[i].Enabled = true;
            }
            btnCloseAll.Enabled = true;
            btnOpenAll.Enabled = true;
        }
    }
}
