﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace Perfusjonssytem
{
    public delegate void IncrementDelegate();

    [Serializable]
    public partial class SeqIndicator : UserControl
    {
        List<Segment> segments;

        Panel indicator;
        Color indicatorActiveColor = Color.FromArgb(20, 108, 18);
        Color indicatorInactiveColor = Color.FromArgb(115, 23, 20);
        const double INDICATOR_WIDTH = 10;

        Color defaultBackColor;
        Color colorA = Color.FromArgb(30, 129, 176);
        Color colorB = Color.FromArgb(21, 76, 121);
        const int TIMER_INTERVAL = 16;
        Stopwatch indicatorSW = new Stopwatch();
        Timer indicatorTickTimer = new Timer();
        
        int totalDuration = 0;
        double subPixelIncrement;
        double indicatorProgress;

        static ToolTip toolTip = new ToolTip();

        bool stop = false;

        public SeqIndicator()
        {
            InitializeComponent();

            indicatorTickTimer.Tick += indicatorTimer_tick;

            segments = new List<Segment>();

            indicator = new Panel();

            BackColor = Color.FromArgb(66, 66, 69);
            defaultBackColor = BackColor;
        }

        public void ClearSegments()
        {
            segments.Clear();
            Redraw();
        }

        public void RemoveSegmentAt(int index)
        {
            segments.RemoveAt(index);
            Redraw();
        }

        public void AddSegment(string segmentName, int segmentDuration)
        {
            Segment seg = new Segment();

            seg.label.TextAlign = ContentAlignment.MiddleCenter;
            seg.label.Text = segmentName;
            seg.label.AutoSize = true;
            toolTip.SetToolTip(seg.panel, seg.label.Text);
            seg.duration = segmentDuration;

            segments.Add(seg);
            Redraw();
        }

        void Redraw()
        {
            totalDuration = 0;
            for (int i = 0; i < segments.Count; i++)
            {
                totalDuration += segments[i].duration;
            }

            int width = 1;
            int xLocationOff = 0;
            bool colorPickerA = true;
            this.Controls.Clear();
            for (int i = 0; i < segments.Count; i++)
            {
                // Alternate segment colors
                if (colorPickerA) segments[i].panel.BackColor = colorA;
                else segments[i].panel.BackColor = colorB;
                colorPickerA = !colorPickerA;


                width = Convert.ToInt32((Convert.ToDouble(segments[i].duration) / Convert.ToDouble(totalDuration)) * this.Width);
                segments[i].panel.Size = new Size(width, this.Height);
                segments[i].panel.Location = new Point(xLocationOff, 0);

                // Add label to controls to make it auto-resize, then remove if it's still to large
                this.Controls.Add(segments[i].label);
                if (segments[i].label.Width > segments[i].panel.Width) this.Controls.RemoveAt(this.Controls.Count - 1);

                segments[i].label.Location = new Point(xLocationOff + (segments[i].panel.Width - segments[i].label.Width) / 2, (segments[i].panel.Height - segments[i].label.Height) / 2);
                segments[i].label.BackColor = segments[i].panel.BackColor;

                xLocationOff += width;

                this.Controls.Add(segments[i].panel);

                segments[i].label.BringToFront();
            }

            if (segments.Count > 0) this.BackColor = segments.Last().panel.BackColor;
            else this.BackColor = defaultBackColor;
        }

        void SetIndicator()
        {
            indicator.Width = (int)INDICATOR_WIDTH;
            indicator.Height = this.Height;
            indicator.Location = new Point(-(int)(INDICATOR_WIDTH / 2.0), 0);
            indicatorProgress = -INDICATOR_WIDTH / 2.0;
            indicator.BackColor = indicatorActiveColor;
            this.Controls.Add(indicator);
            indicator.BringToFront();
        }

        public void Start()
        {
            if (segments.Count > 0)
            {
                stop = false;
                SetIndicator();
                indicatorTickTimer.Start();
                indicatorSW.Start();
            }
        }

        public void Stop()
        {
            indicatorTickTimer.Enabled = false;
            indicatorSW.Reset();
            indicator.BackColor = indicatorInactiveColor;
            stop = true;
        }

        public void indicatorTimer_tick(object sender, EventArgs e)
        {
            double timeRatio = Convert.ToDouble(indicatorSW.ElapsedMilliseconds) / Convert.ToDouble(totalDuration);

            int indicatorMiddleLocation = (int)(timeRatio * this.Width);

            // Draw from left part of indicator
            indicator.Location = new Point(indicatorMiddleLocation - (int)(INDICATOR_WIDTH / 2.0), 0);

            if (indicatorMiddleLocation >= this.Width) Stop();
        }

        private class Segment
        {
            public Panel panel;
            public Label label;
            public int duration;

            public Segment()
            {
                panel = new Panel();
                label = new Label();
            }
        }
    }
}
