﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Microsoft.WindowsAPICodePack.Dialogs;

namespace Perfusjonssytem
{
    public partial class SettingsForm : Form
    {
        PerfusionForm perfusinForm;
        List<ComboBox> cbSubstanceList;

        internal List<ComboBox> CbSubstanceList { get => cbSubstanceList; set => cbSubstanceList = value; }
        internal PictureBox PbArduinoStatusSet { get => pbArduinoStatusSet; set => pbArduinoStatusSet = value; }

        public SettingsForm(PerfusionForm pf)
        {
            InitializeComponent();
            cbSubstanceList = new List<ComboBox>();
            
            this.TopLevel = false;

            perfusinForm = pf;

            cbSubstanceList.Add(cbCh1Substance);
            cbSubstanceList.Add(cbCh2Substance);
            cbSubstanceList.Add(cbCh3Substance);
            cbSubstanceList.Add(cbCh4Substance);

            Settings.ReadKnownSubstancesFromFile();

            for (int i = 0; i < cbSubstanceList.Count; i++)
            {
                cbSubstanceList[i].DataSource = new BindingSource { DataSource = Settings.KnownSubstancesList };
            }

            SelectPreviousCbChSubstances();

            cbRemoveSubstance.DataSource = new BindingSource { DataSource = Settings.KnownSubstancesList };

            COM.ConnectedDel += UpdatebtnConnect;
            COM.DisconnectedDel += UpdatebtnConnect;

            SelectPreviousLogFolder();

            cbRemoveSubstance_SelectedIndexChanged(null, EventArgs.Empty);
        }
        private void SelectPreviousCbChSubstances()
        {
            List<string> selectedSubstances = Settings.ReadSelectedChSubstances();
            for (int i = 0; i < selectedSubstances.Count; i++)
            {
                cbSubstanceList[i].SelectedItem = selectedSubstances[i];
            }
        }
        private void SelectPreviousLogFolder()
        {
            string buffer = Settings.ReadPreviousLogFolder();
            if (buffer != null && buffer != "")
            {
                Log.LogFilePathFolder = buffer;
                txtLogFolder.Text = buffer.Substring(buffer.IndexOf("Logs"));
            }
        }
        private void btnConnect_Click(object sender, EventArgs e)
        {
            if (COM.ArduinoPort == null || (COM.ArduinoPort != null && !COM.ArduinoPort.IsOpen))
            {
                COM.ConnectToArduino();
            }
        }
        private void btnSelectFolder_Click(object sender, EventArgs e)
        {
            CommonOpenFileDialog dialog = new CommonOpenFileDialog();
            dialog.IsFolderPicker = true;
            dialog.InitialDirectory = AppDomain.CurrentDomain.BaseDirectory + "Logs";
            if (dialog.ShowDialog() == CommonFileDialogResult.Ok)
            {
                if (dialog.FileName.Contains(AppDomain.CurrentDomain.BaseDirectory + "Logs"))
                {
                    txtLogFolder.Text = dialog.FileName.Substring(dialog.FileName.IndexOf("Logs"));
                    Log.LogFilePathFolder = dialog.FileName;
                }
                else
                {
                    MessageBox.Show("The chosen folder must be within the \"Logs\" folder");
                }
            }
        }
        private void btnRemoveSubstance_Click(object sender, EventArgs e)
        {
            if (cbRemoveSubstance.SelectedItem.ToString() != "None")
            {
                string[] selectedItems = new string[cbSubstanceList.Count];
                for (int i = 0; i < cbSubstanceList.Count; i++)
                {
                    selectedItems[i] = cbSubstanceList[i].SelectedItem.ToString();
                }

                Settings.KnownSubstancesList.Remove(cbRemoveSubstance.SelectedItem.ToString());

                for (int i = 0; i < cbSubstanceList.Count; i++)
                {
                    if (Settings.KnownSubstancesList.Contains(selectedItems[i])) cbSubstanceList[i].SelectedItem = selectedItems[i];
                }

                perfusinForm.SequenceForm.UpdateGUI();
            }
        }
        private void cbRemoveSubstance_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cbRemoveSubstance.SelectedItem == null || cbRemoveSubstance.SelectedItem.ToString() == "None")
            {
                btnRemoveSubstance.Enabled = false;
            }
            else
            {
                btnRemoveSubstance.Enabled = true;
            }
        }
        private void UpdatebtnConnect()
        {
            if (COM.Connected)
            {
                btnConnect.Enabled = false;
            }
            else
            {
                btnConnect.Enabled = true;
            }
        }
        public void UpdateKnownSubstanceList()
        {
            for (int i = 0; i < cbSubstanceList.Count; i++)
            {
                if (cbSubstanceList[i].Text.ToUpper().Contains("NONE") && cbSubstanceList[i].Items.Count != 0) cbSubstanceList[i].SelectedIndex = 0;
                else if (cbSubstanceList[i].SelectedItem == null && !Settings.KnownSubstancesList.Contains(cbSubstanceList[i].Text) && !String.IsNullOrWhiteSpace(cbSubstanceList[i].Text)) // check for new substances
                {
                    Settings.KnownSubstancesList.Add(cbSubstanceList[i].Text);
                    cbSubstanceList[i].SelectedIndex = cbSubstanceList[i].Items.Count - 1; // select new substance
                }
                else if (!String.IsNullOrWhiteSpace(cbSubstanceList[i].Text))
                {
                    cbSubstanceList[i].SelectedIndex = Settings.KnownSubstancesList.IndexOf(cbSubstanceList[i].Text); // set the substance to the allready existing substance with the same name
                }
                else if (cbSubstanceList[i].Items.Count != 0) // non valid string input, select first element if there is one
                {
                    cbSubstanceList[i].SelectedIndex = 0;
                }
            }
        }
        private void cbSubstance_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateKnownSubstanceList();
            if (perfusinForm.SequenceForm != null) perfusinForm.SequenceForm.UpdateGUI();
        }

        private void SettingsForm_Click(object sender, EventArgs e)
        {
            cbSubstance_SelectedIndexChanged(this, EventArgs.Empty);
        }
    }
}
